/**
  * @file TemperatureSum.cpp
  * @author E.Casellas-(The RECORD team -INRA )
  *
  * Copyright (C) 2009 INRA
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */

#include <vle/extension/DifferenceEquation.hpp>
#include <vle/devs/DynamicsDbg.hpp>

namespace vd = vle::devs;
namespace ve = vle::extension;
namespace vv = vle::value;
using namespace std;

namespace gluePhysic {
	class TemperatureSum : public ve::DifferenceEquation::Multiple
	{
	public:
		TemperatureSum(
		   const vd::DynamicsInit& atom,
		   const vd::InitEventList& evts)
		    : ve::DifferenceEquation::Multiple(atom, evts)
		{
		    maxTemp = (evts.exist("maxTemp")) ? vv::toDouble(evts.get("maxTemp")) : 500.0;  
		    minTemp = (evts.exist("minTemp")) ? vv::toDouble(evts.get("minTemp")) : 0.0;  
		    baseTemp = (evts.exist("baseTemp")) ? vv::toDouble(evts.get("baseTemp")) : 0.0;  
		    maxEff = (evts.exist("maxEff")) ? vv::toDouble(evts.get("maxEff")) : 1.0;  

		    ThermalTime = createVar("ThermalTime");
		    Active = createVar("Active");
		    TmoyEff = createVar("TmoyEff");
		    MaxTemp = createVar("MaxTemp");
		    MinTemp = createVar("MinTemp");
		    BaseTemp = createVar("BaseTemp");
		    MaxEff = createVar("MaxEff");
		    
		    Tmin = createSync("Tmin");
		    Tmax = createSync("Tmax");
		}

		virtual ~TemperatureSum() {};

		virtual void compute(const vd::Time& /*time*/)
		{
			Active = Active(-1);
			MaxTemp = MaxTemp(-1);
			MinTemp = MinTemp(-1);
			BaseTemp = BaseTemp(-1);
			MaxEff = MaxEff(-1);
			
			if (Active() ==0.0)
				TmoyEff = 0.0;		
			else
			{
				double TminLimited = min( MaxTemp(), max( MinTemp(), Tmin() ) );
				double TmaxLimited = min( MaxTemp(), max( MinTemp(), Tmax() ) );
				double TmoyLimited = (TminLimited + TmaxLimited) / 2;
				TmoyEff = max( 0.0, MaxEff() * (TmoyLimited - BaseTemp() ) );
			}
			ThermalTime = ThermalTime(-1) + TmoyEff();
		}

		virtual void initValue(const vd::Time& /*time*/)
		{
			ThermalTime = 0.0;
			TmoyEff = 0.0;
			Active = 0.0;
			MaxTemp = maxTemp;
			MinTemp = minTemp;
			BaseTemp = baseTemp;
			MaxEff = maxEff;
		}

	private:
		double maxTemp;
		double minTemp;
		double baseTemp;
		double maxEff;
		Var ThermalTime;
		Var Active;
		Var TmoyEff;
		Var MinTemp;
		Var MaxTemp;
		Var BaseTemp;
		Var MaxEff;
		Sync Tmin;
		Sync Tmax;
	};
} // namespace gluePhysic
DECLARE_DYNAMICS_DBG(gluePhysic::TemperatureSum)
