/**
  * @file DecFSA.cpp
  * @author E.Casellas-(The RECORD team -INRA )
  *
  * Copyright (C) 2009 INRA
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
  
#include <vle/extension/fsa/Statechart.hpp>
#include <vle/extension/DifferenceEquation.hpp>
#include <vle/utils/Trace.hpp>
#include <vle/utils/i18n.hpp>
#include <vle/devs/DynamicsDbg.hpp>

using namespace std;
using namespace vle;
namespace ve = vle::extension;
namespace vd = vle::devs;

namespace gluePhysic {
	enum DecState {DEC_BARE_SOIL, DEC_PLANTED, DEC_STADE, DEC_HARVEST};
	class DecFSA: public vle::extension::fsa::Statechart
	{
	public:
		DecFSA(const devs::DynamicsInit& atom,
				const devs::InitEventList& events) :
				Statechart(atom, events), Temp(0)
		{
		    states(this) << DEC_BARE_SOIL << DEC_PLANTED << DEC_STADE << DEC_HARVEST;

			transition(this, DEC_BARE_SOIL, DEC_PLANTED)
				<< guard(&DecFSA::sowing)
				<< send(&DecFSA::out_sowing);
		    transition(this, DEC_PLANTED, DEC_STADE)
		        << guard(&DecFSA::stade)
		        << send(&DecFSA::out_stade);
		    transition(this, DEC_STADE, DEC_HARVEST)
		        << guard(&DecFSA::harvesting)
		        << send(&DecFSA::out_harvesting);

		    initialState(DEC_BARE_SOIL);        
		    
		   	 eventInState(this, "Tmin", &DecFSA::inTemp)
			>> DEC_BARE_SOIL >> DEC_PLANTED >> DEC_STADE >> DEC_HARVEST;
		}
		
		virtual ~DecFSA() {};
		
		virtual vle::value::Value* observation(const vd::ObservationEvent& /*event*/) const
		{ return vle::value::Integer::create(currentState()); }
		
	private :
		//Sowing 
		bool sowing(const vd::Time& julianDay)
		{ return (vle::utils::DateTime::dayOfYear(julianDay) == 30); }
		
		void out_sowing(const vd::Time& /*julianDay*/, vd::ExternalEventList& output) const
		{
		    output << (ve::DifferenceEquation::Var("Active") = 1.0);
		    output << (ve::DifferenceEquation::Var("ThermalTime") = 0.0);
		}
		
		//Stade
		bool stade(const vd::Time& julianDay)
		{ return (vle::utils::DateTime::dayOfYear(julianDay) == 150); }
		
		void out_stade(const vd::Time& /*julianDay*/, vd::ExternalEventList& output) const
		{
		    output << (ve::DifferenceEquation::Var("BaseTemp") = 1.0);
		    output << (ve::DifferenceEquation::Var("MaxTemp") = 20.0);
		    output << (ve::DifferenceEquation::Var("MinTemp") = 0.0);
		    output << (ve::DifferenceEquation::Var("MaxEff") = 0.9);
		}
		
		//Harvesting
		bool harvesting(const vd::Time& julianDay)
		{ return (vle::utils::DateTime::dayOfYear(julianDay) == 300); }
		
		void out_harvesting(const vd::Time& /*julianDay*/, vd::ExternalEventList& output) const
		{
		    output << (ve::DifferenceEquation::Var("Active") = 0.0);
		    output << (ve::DifferenceEquation::Var("ThermalTime") = 0.0);
		}

	   void inTemp(const vd::Time& /* time */, const vd::ExternalEvent* event)
	   { Temp << ve::DifferenceEquation::Var("Tmin", event); }

		//consition variables
		double Temp;
	};
}
DECLARE_DYNAMICS_DBG(gluePhysic::DecFSA);
